/** 
 *   Thermometer logger novus
 *
 *   (c) Copyright 2005 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/


#define _GNU_SOURCE

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "modbus.h"
#include "logger_novus.h"
#include "my_stdio.h"


int logger_init(struct logger_nv *lnv, char *device)
{
	if ((lnv->fd = modbus_open(device)) > 0) {
		logger_get_conf(lnv);
	}

	return lnv->fd;
}


int logger_close(struct logger_nv *lnv)
{
	modbus_close(lnv->fd);

	return 0;
}


static double logger_scale(int type, int val)
{
	double result;

	switch(type) {
	case PT100:
		result = (double) val / 10.0;
		break;
	case THERMOCOUPLE_J:
	case THERMOCOUPLE_K:
	case THERMOCOUPLE_T:
	case THERMOCOUPLE_E:
	case THERMOCOUPLE_N:
	case THERMOCOUPLE_R:
	case THERMOCOUPLE_S:
	case THERMOCOUPLE_B:
		result = (double) val;
		break;
	default:
		printf("unknown input devece 0x%x\n", type);
		result = (double) val;
		break;
	}

	return result;
}

#define VAL_IN_ADDR(buff, addr) (*(buff+(addr << 1)) << 8) | *(buff+((addr << 1)| 1))

#if 0
int val_in_addr(unsigned char *buff, int addr)
{
	return ((buff[addr << 1] << 8) | buff[(addr << 1) + 1]);
}
#endif

int logger_adq(struct logger_nv *lnv)
{
	int result;
	int len, i;
	unsigned char pk[MODBUS_PACKET_LENGTH];
	int length = 8;

	len = modbus_make_pk(pk, LOGGER_SLAVE, 3, 0, length);
	result = modbus_write_pk(lnv->fd, pk, len);
	if (result > 0) {
		result = 0;
		len = modbus_read_pk(lnv->fd, pk, 3 + (length << 1) + 2);
		if (len > 0 ) {
			for(i=0;i<length;i++) {
				if (lnv->thr[i].active) {
					lnv->thr[i].val = 
						logger_scale(lnv->thr[i].type, 
						     VAL_IN_ADDR(pk+3,i));
					/*		 
							     pk[3+(2*i)] << 8| 
							     pk[3+(2*i)+1]);
					*/
					result++;
				}
			}
		}else{
			result = len;
		}
	}

	return result;
}


int logger_set_reg(struct logger_nv *lnv, int reg, int val)
{
	int result;
	int len;
	unsigned char pko[MODBUS_PACKET_LENGTH];
	unsigned char pki[MODBUS_PACKET_LENGTH];

	len = modbus_make_pk(pko, lnv->slave, 0x06, reg, val);
	result = modbus_write_pk(lnv->fd, pko, len);
	if (result > 0) {
		result = -1;
		len = modbus_read_pk(lnv->fd, pki, len);
		if (len > 0 ) {
			lnv->map[reg << 1] = (val >> 8) & 0xff;
			lnv->map[(reg << 1) | 1 ] = val & 0xff;
			result = memcmp(&pko, &pki, len);
		}
	}
		
	return result;
}


int logger_get_reg(struct logger_nv *lnv, int reg, int *val)
{
	int result;
	int len;
	unsigned char pk[MODBUS_PACKET_LENGTH];

	len = modbus_make_pk(pk, lnv->slave, 0x03, reg, 0x01);
	result = modbus_write_pk(lnv->fd, pk, len);
	if (result > 0) {
		result = -1;
		len = modbus_read_pk(lnv->fd, pk, 7);
		if (len > 0 ) {
			*val = VAL_IN_ADDR(pk+3,0);
			result = 0;
		}
	}

	return result;


}



static int _get_conf(struct logger_nv *lnv, int addr, int length)
{
	int result;
	int len;
	unsigned char pk[MODBUS_PACKET_LENGTH];

	len = modbus_make_pk(pk, lnv->slave, 0x03, addr, length); 
	result = modbus_write_pk(lnv->fd, pk, len);
	if (result > 0) {
		result = -1;
		len = modbus_read_pk(lnv->fd, pk, 3 + (length << 1) + 2);
		if (len > 0 ) {
			memcpy(lnv->map+(addr << 1), pk + 3, length << 1);
			result = 0;
		}
	}

	return result;


}


int logger_get_conf(struct logger_nv *lnv)
{
	int i, val;
	int length = (MODBUS_PACKET_LENGTH - 5) >> 1;
	int reg = 0;
	/* (MODBUS_PACKET_LENGTH - 5) >> 1 = 0x7d: 125, la longitud
	   total es: 3 + 125 *2 + 2 = 255 */

	if (_get_conf(lnv, reg, length) != 0)
		return -1;
	reg = length;
	length = LOGGER_CONF_REG_LENGTH - length; 
	if (_get_conf(lnv, reg, length) != 0)
		return -1;

	lnv->num_act_thr = 0;
	memcpy(lnv->label, lnv->map+(76 << 1), 16); 
	for (i=0; i < 8;i++) {
		/* Configuracion de alarmas y abilitacion de canales */
		val = VAL_IN_ADDR(lnv->map, (17 + i));
		lnv->thr[i].al_type = val >> 8;
		lnv->thr[i].active = (val & CHANNEL_ENABLE) >> 7;
		lnv->thr[i].alarm = VAL_IN_ADDR(lnv->map, (26 + i));
		if(val & LABEL_ENABLE) {
			memcpy(lnv->thr[i].label, 
			       lnv->map+((84 << 1) + (i << 3)), 8);
			lnv->thr[i].type = PT100; /* FIXME */
			lnv->num_act_thr++;
		} else {
			lnv->thr[i].label[0] = 0;
		}
	}
		
	return 0;

}


int logger_relay(struct logger_nv *lnv, int relay, int state)
{
	int result;
	int len;
	unsigned char pko[MODBUS_PACKET_LENGTH];
	unsigned char pki[MODBUS_PACKET_LENGTH];

	if (state > 0) state = 0xff00;  
	else state = 0;
	len = modbus_make_pk(pko, LOGGER_SLAVE, 5, relay, state);
	result = modbus_write_pk(lnv->fd, pko, len);
	if (result > 0) {
		result = -1;
		len = modbus_read_pk(lnv->fd, pki, len);
		if (len > 0 ) { 
			result = memcmp(&pko, &pki, len);
		}
	}

	return result;
}


int logger_relay_on(struct logger_nv *lnv, int relay)
{
	return logger_relay(lnv, relay, 1); 
}


int logger_relay_off(struct logger_nv *lnv, int relay)
{
	return logger_relay(lnv, relay, 0); 
}


int logger_enable_relay(struct logger_nv *lnv, int relay)
{
	int val;
	int result = -1;
	
	if (logger_get_reg(lnv, 25, &val)) {
		switch(relay) {
		case 0:
			val |= RELAY_0_ENABLE;
			break;
		case 1:
			val |= RELAY_1_ENABLE;
			break;
		default:
			SYSLOG(LOG_ERR,"unknow relay %d", relay);
			return -1;
		}
		result = logger_set_reg(lnv, 25, val);

	}

	return result;
} 


int logger_disable_relay(struct logger_nv *lnv, int relay)
{
	int val;
	int result = -1;
	
	if (logger_get_reg(lnv, 25, &val)) {
		switch(relay) {
		case 0:
			val &= (0xff ^ RELAY_0_ENABLE);
			break;
		case 1:
			val &= (0xff ^ RELAY_1_ENABLE);
			break;
		default:
			SYSLOG(LOG_ERR,"unknow relay %d", relay);
			return -1;
		}
		result = logger_set_reg(lnv, 25, val);

	}

	return result;

}




/********************/

int logger_test(int fd, struct logger_input *tn, int length)
{
	int result;
	int len, i;
	unsigned char pk[MODBUS_PACKET_LENGTH];

	memset(pk, 0, 8);
	length = 0x0f+13;
	len = modbus_make_pk(pk, LOGGER_SLAVE, 3, 0x08, length);
	result = modbus_write_pk(fd, pk, len);
	if (result > 0) {
		result = 0;
		len = modbus_read_pk(fd, pk, 3 + length * 2 + 2);
		if (len > 0 ) { 
			for(i=0;i<len;i++) {
				printf("%x ", pk[i]);
				result++;
				
			}
			printf("\n");
//0                    1                       2                     3
//0 1  2 3 4 5 6 7 8 9 0 1 2 3  4  5  6 7 8 9  0 1  2 3  4 5 6 7 8 9 0 1 2  3
//1 3 1e 0 8 0 0 0 0 0 0 0 0 e 21 10 16 6 7 6 88 0 88 0 88 0 0 0 0 0 0 0 0 4a 8a
			printf("%02d:%02d:%02d - %04d/%02d/%02d\n",
			       pk[15+2],pk[14+2],pk[13+2], 
			       pk[18+2]+1998, pk[17+2],pk[16+2]);

		}else{
			result = len;
		}
	}

	return result;
}


int logger_test2(int fd)
{
	int result;
	int len, i, length;
	unsigned char pk[MODBUS_PACKET_LENGTH];

	verbose_flag =0xf;

	memset(pk, 0, 8);
	length = 1;
	len = modbus_make_pk(pk, LOGGER_SLAVE, 5, 0x0000, 0xff00);
	result = modbus_write_pk(fd, pk, len);
	if (result > 0) {
		result = 0;
		len = modbus_read_pk(fd, pk, len);
		if (len > 0 ) { 
			for(i=0;i<len;i++) {
				printf("%x ", pk[i]);
				result++;
				
			}
			printf("\n");
		}else{
			result = len;
		}
	}

	return result;
}
